
/*
   This file is part of AirFart.

   AirFart is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   AirFart is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with AirFart; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef FRAMESNIFFER_H
#define FRAMESNIFFER_H

/*
 *  This is a test of CVS
 */

/* system includes */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys/socket.h>
#include <linux/if_ether.h> // gives us ETH_P_ALL
#include <netinet/in.h>

/*Unused includes */
//#include <sys/types.h>
//#include <sys/ioctl.h>
//#include <sys/utsname.h>
//#include <net/if.h>
//#include <string.h>
//#include <net/if_arp.h>
//#include <errno.h>

/* My header file */
#include "stats.h"
#include "address.h"

/* Shall we dump some helpful debugging output? */
//#define DEBUG_SNIFFER 1

/* the biggest frame we can read */
#define MAX_BUFFER_SIZE 4000

/* Offsets in the AVS header (that wlan-ng puts on each frame) */
#define DOT11_IDENT_OFFSET 0
#define AVS_HEADER_LEN_OFFSET 4
#define SSI_TYPE_OFFSET 44
#define SSI_SIGNAL_OFFSET 48
#define SSI_NOISE_OFFSET 52
#define ETH_HEADER_OFFSET 64
#define DST_ETH_ADDR_OFFSET 68
#define SRC_ETH_ADDR_OFFSET 74

/* Special hack offsets for empty frames */
// UNDER CONSTRUCTION
#define EMPTY_FRAME_SRC_ETH_ADDR_OFFSET -1
#define EMPTY_FRAME_LEN 14

/* Some byte lenghts */
#define ETH_ADDR_LEN 6
#define AVS_HEADER_LEN 64

/* Channel to sniff */
#define DEFAULT_CHANNEL 6

/* 802.11 value (first word of every wlan packet) */
#define DOT11_IDENT { 0x80, 0x21, 0x10, 0x01 }
#define DOT11_IDENT_LEN 4

/* Class responsible for sniffing ethernet frames, extracting their signal strength
 * and passing them to the statistics unit for analysis */
class frame_sniffer
{
	public:
		/* Sets our pointer to an initialized stats collector */
		void set_stats_collector( Stats *s );
		/* Gentlemen, start your engines */
		void start_sniffing();

	private:
		/* The socket we open to the ethernet device */
		int skt;
		/* Our pointer to the stats collector, set with method above*/
		Stats *stats;
		/* We dump every frame into this buffer */
		unsigned char msgbuf[MAX_BUFFER_SIZE];
		/* for binding the socket to the ethernet device */
		struct sockaddr_ll
		{
			unsigned short sll_family;
			unsigned short sll_protocol;
			int sll_ifindex;
			unsigned short sll_hatype;
			unsigned char sll_pkttyhpe;
			unsigned char sll_halen;
			unsigned char sll_addr[8];
		};

		/* Prettily prints an ethernet address */
		void print_eth_addr( unsigned char *eth_addr );
		/* Buidls a socket and sets skt to it*/
		int set_socket();
		/* Binds skt to the ethernet device */
		int bind_socket();
		/* Puts the wireless card in monitor mode (uses wlanctl-ng) */
		int go_monitor_mode();
};

#endif

