
/*
   This file is part of AirFart.

   AirFart is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   AirFart is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with AirFart; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <pthread.h>
#include <semaphore.h>

#include <sys/time.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  gtk_object_set_data_full (GTK_OBJECT (component), name, \
    gtk_widget_ref (widget), (GtkDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  gtk_object_set_data (GTK_OBJECT (component), name, widget)
  
#define LIST_VIEW_ICON_HEIGHT 15
#define LIST_VIEW_ICON_MAX_WIDTH 100

GdkRectangle treeview_rectangle;

  sem_t semaphore;
  GtkWidget *treeview;
  int active_secs;
	GtkWidget *active1;
	GtkWidget *active2;
	GtkWidget *active3;
	GtkWidget *active4;
	GtkWidget *active5;

void uncheck_other_actives(int sec){
	gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM(active1),
                                             (sec == 1));
	gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM(active2),
                                             (sec == 2));
	gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM(active3),
                                             (sec == 3));
	gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM(active4),
                                             (sec == 4));
	gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM(active5),
                                             (sec == 5));
}

GdkPixbuf*
get_icon (char* file)
{

	GdkPixbuf *pixbuf = NULL;
	GdkPixbuf *pixbuf_icon = NULL;
	int width, height;
	double factor;
	//gdk_threads_enter();
	pixbuf = gdk_pixbuf_new_from_file (file, NULL);

	if (pixbuf == NULL){
	  printf("my %s pixfuf is null\n",file);
	 return NULL;
	}

	width = gdk_pixbuf_get_width (pixbuf);
	height = gdk_pixbuf_get_height (pixbuf);
	factor = (double) LIST_VIEW_ICON_HEIGHT / (double) height;
	pixbuf_icon = gdk_pixbuf_scale_simple (pixbuf, (int) (width * factor),
					       LIST_VIEW_ICON_HEIGHT,
					       GDK_INTERP_BILINEAR);

	g_object_unref (pixbuf);
	//gdk_threads_leave();

	//printf("my %s pixfuf is working\n",file);
	return pixbuf_icon;
}

GdkPixbuf*
scale_icon_width (GdkPixbuf* pixbuf, int percentage)
{
	GdkPixbuf* pixbuf_icon;
	//gdk_threads_enter();
	//(100/LIST_VIEW_ICON_MAX_WIDTH)),
	pixbuf_icon = gdk_pixbuf_scale_simple (pixbuf, (percentage * 2),
					       LIST_VIEW_ICON_HEIGHT,
					       GDK_INTERP_BILINEAR);

	g_object_unref (pixbuf);
	//gdk_threads_leave();
	printf("I just scaled it to %d\n",percentage);
	return pixbuf_icon;
}

enum
{
  COLUMN_ETH_ADDR_STRING,
  COLUMN_STRENGTH,
  COLUMN_STRENGTH_BAR,
  COLUMN_ACTIVE,
  COLUMN_ACTIVE_LIGHT,
  COLUMN_TIME_STAMP,
  NUM_COLUMNS
};

void set_active_secs(int secs){
	printf("active secs was: %d\n",active_secs);
	active_secs = secs;
	printf("set active secs: %d\n",active_secs);
}
int get_active_secs(){
	return active_secs;
}
void* deactivate (void* arg){

	GtkTreeIter iter;
	long time;
	glong time_stamp;
	gboolean active;
	struct timeval tv;
	GtkTreeModel* model;
	GdkPixbuf* light;
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));

	while (1){
		sem_wait(&semaphore);
		gettimeofday(&tv,NULL);
		time = tv.tv_sec;
		if (gtk_tree_model_get_iter_first (model, &iter)){//there is something in the tree
			do{
				gtk_tree_model_get(model, &iter, COLUMN_TIME_STAMP, &time_stamp, COLUMN_ACTIVE, &active, -1);
				if (active && (time_stamp + get_active_secs() <= time)){//we should deactivate this entry
					gtk_tree_model_get(model, &iter, COLUMN_ACTIVE_LIGHT, &light, -1);
					printf("\n\n\n\nTurning off a light\n\n\n\n");
					light = get_icon("graphics/offlight.png");
					gtk_list_store_set (GTK_LIST_STORE (model), &iter,
								COLUMN_ACTIVE_LIGHT, light,
								COLUMN_ACTIVE, FALSE,
								-1);//update the percentage
				}
			}while (gtk_tree_model_iter_next(model, &iter));
		}//end if
		gdk_threads_enter();
		//gtk_tree_view_get_visible_rect  (GTK_TREE_VIEW (treeview), &treeview_rectangle);
		//gtk_widget_draw(treeview,&treeview_rectangle);
		gdk_threads_leave();
		sem_post(&semaphore);
		sleep(1);
	}//end while
}

static GtkTreeModel *create_model (void)
{
	//creates a new list store
	GtkListStore *store;
	store = gtk_list_store_new (NUM_COLUMNS,
			      G_TYPE_STRING,
			      G_TYPE_INT,
			      GDK_TYPE_PIXBUF,
			      G_TYPE_BOOLEAN,
			      GDK_TYPE_PIXBUF,
			      G_TYPE_LONG);
  	return GTK_TREE_MODEL (store);
}


void print_eth_addr2( unsigned char *eth_addr )
{
	printf( "%x:%x:%x:%x:%x:%x",
		eth_addr[0],eth_addr[1],eth_addr[2],
		eth_addr[3],eth_addr[4],eth_addr[5] );
}


void update_treeview(unsigned char* eth_addr, char* ss_id, gint strength){


	GdkPixbuf* strengthbar;
	GdkPixbuf* light;
	/* Used to create a new row in the tree view */
	gchar eth_addr_string[255];

	long time;
	GtkTreeIter iter;
	gchar* eth_addr_gstr;
	GtkTreeModel *model;
	struct timeval tv;
	gettimeofday(&tv,NULL);
	time = tv.tv_sec;
	gboolean made_change = FALSE;
	gboolean active;
	model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
	printf("we are in update treeview !!!\n");
	//printf("The time stamp for this UPDATE is %d\n",time);
	gdk_threads_enter();
	light = get_icon("graphics/onlight.png");
	gdk_threads_leave();

	sem_wait(&semaphore);
	//printf( "eth_addr: " );
	//print_eth_addr2( eth_addr );
	//printf( "\n" );
	g_snprintf( eth_addr_string, 255, "%x:%x:%x:%x:%x:%x",
		eth_addr[0], eth_addr[1], eth_addr[2],
		eth_addr[3], eth_addr[4], eth_addr[5] );

	if (gtk_tree_model_get_iter_first (model, &iter)){//there is something in the tree
		//printf("There is stuff in the tree\n");
		do{
			gtk_tree_model_get(model, &iter, COLUMN_ETH_ADDR_STRING, &eth_addr_gstr, -1);
			//printf( "Comparing incoming %s with model %s...\n", eth_addr_string, eth_addr_gstr );
			if (strcmp(eth_addr_gstr,eth_addr_string)==0){//we already have this ethernet address
				//printf("I found a match!\n");
				gdk_threads_enter();
				gtk_tree_model_get(model,&iter,COLUMN_STRENGTH_BAR,&strengthbar, COLUMN_ACTIVE, &active, -1);
				gdk_threads_leave();
				gdk_threads_enter();
				strengthbar = scale_icon_width(strengthbar,strength);
				gdk_threads_leave();
				gtk_list_store_set (GTK_LIST_STORE (model), &iter,
							COLUMN_STRENGTH, strength,
							COLUMN_STRENGTH_BAR,strengthbar,
							COLUMN_ACTIVE, TRUE,
							COLUMN_TIME_STAMP, time,
							-1);//update the percentage
				if (!active){
					gtk_list_store_set (GTK_LIST_STORE (model), &iter,
							COLUMN_ACTIVE_LIGHT, light,
							-1);//update the percentage
				}
				made_change = TRUE;
			}
		}while (!made_change && gtk_tree_model_iter_next(model, &iter));
	}
	if (!made_change){
		printf("I still haven't found one yet\n");

		gdk_threads_enter();
		strengthbar = get_icon("graphics/statusbar.png");
		//printf("I am in creating status bar\n");
		gdk_threads_leave();
		gdk_threads_enter();
		//printf("I am in changing width bar\n");
		strengthbar = scale_icon_width(strengthbar,strength);
		gdk_threads_leave();
		gtk_list_store_append (GTK_LIST_STORE (model), &iter);
		gtk_list_store_set (GTK_LIST_STORE (model), &iter,
				COLUMN_ETH_ADDR_STRING, eth_addr_string,
				COLUMN_STRENGTH, strength,
				COLUMN_STRENGTH_BAR,strengthbar,
				COLUMN_ACTIVE_LIGHT, light,
				COLUMN_ACTIVE, TRUE,
				COLUMN_TIME_STAMP, time,
				-1);
	}//end
	gdk_threads_enter();
	//gtk_tree_view_get_visible_rect  (GTK_TREE_VIEW (treeview), &treeview_rectangle);
	//gtk_widget_draw(treeview,&treeview_rectangle);
	gdk_threads_leave();
	sem_post(&semaphore);
}

static void add_columns ()
{
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *column;

  /* column for ethernet address */
  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("Eth Addr",
						     renderer,
						     "text",
						     COLUMN_ETH_ADDR_STRING,
						     NULL);
  /* set this column to a fixed sizing (of 50 pixels) */
  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),
				   GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 150);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_ETH_ADDR_STRING);
  gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);

  /* column for ss id */
  /*
  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes ("SS ID",
						     renderer,
						     "text",
						     COLUMN_SS_ID,
						     NULL);
  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),

				   GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 150);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_SS_ID);
  gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);
  */

  /* column for strength */
  renderer = gtk_cell_renderer_pixbuf_new ();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);//this makes the column aligned left
  column = gtk_tree_view_column_new_with_attributes ("Strength",
						     renderer,
						     "pixbuf",
						     COLUMN_STRENGTH_BAR,
						     NULL);
  /* set this column to a fixed sizing of 100 pixels */
  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),
				   GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 200);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_STRENGTH);
  gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);


  /* column for active */
  renderer = gtk_cell_renderer_pixbuf_new ();
  column = gtk_tree_view_column_new_with_attributes ("Active",
						     renderer,
						     "pixbuf",
						     COLUMN_ACTIVE_LIGHT,
						     NULL);
  /* set this column to a fixed sizing of 50 pixels */
  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN (column),
				   GTK_TREE_VIEW_COLUMN_FIXED);
  gtk_tree_view_column_set_fixed_width (GTK_TREE_VIEW_COLUMN (column), 20);
  gtk_tree_view_column_set_sort_column_id (column, COLUMN_TIME_STAMP);
  gtk_tree_view_append_column (GTK_TREE_VIEW(treeview), column);
}

GtkWidget* create_main_window ()
{
	GtkWidget *main_window;
	GtkWidget *viewport1;
	GtkWidget *vbox1;
	GtkWidget *menubar1;
	GtkWidget *menuitem1;
	GtkWidget *menu1;
	GtkWidget *new1;
	GtkWidget *open1;
	GtkWidget *save1;
	GtkWidget *save_as1;
	GtkWidget *separatormenuitem1;
	GtkWidget *quit1;
	GtkWidget *menuitem2;
	GtkWidget *menu2;
	GtkWidget *copy1;
	GtkWidget *paste1;
	GtkWidget *delete1;
	GtkWidget *menuitem3;
	GtkWidget *menu3;
	GtkWidget *menuitem4;
	GtkWidget *menu4;
	GtkWidget *about1;
	GtkWidget *scrolledwindow2;
	GtkWidget *statusbar1;
	GtkAccelGroup *accel_group;
	GtkTreeModel *datamodel;
	pthread_t threadId;
	//ModelData* md;
	int err;

	sem_init(&semaphore,0,1);
	accel_group = gtk_accel_group_new ();

	main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (main_window), "Airfart");

	viewport1 = gtk_viewport_new (NULL, NULL);
	gtk_widget_show (viewport1);
	gtk_container_add (GTK_CONTAINER (main_window), viewport1);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox1);
	gtk_container_add (GTK_CONTAINER (viewport1), vbox1);

	menubar1 = gtk_menu_bar_new ();
	gtk_widget_show (menubar1);
	gtk_box_pack_start (GTK_BOX (vbox1), menubar1, FALSE, FALSE, 0);

	menuitem1 = gtk_menu_item_new_with_mnemonic ("_File");
	gtk_widget_show (menuitem1);
	gtk_container_add (GTK_CONTAINER (menubar1), menuitem1);

	menu1 = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem1), menu1);

	new1 = gtk_image_menu_item_new_from_stock ("gtk-new", accel_group);
	gtk_widget_show (new1);
	gtk_container_add (GTK_CONTAINER (menu1), new1);

	open1 = gtk_image_menu_item_new_from_stock ("gtk-open", accel_group);
	gtk_widget_show (open1);
	gtk_container_add (GTK_CONTAINER (menu1), open1);

	save1 = gtk_image_menu_item_new_from_stock ("gtk-save", accel_group);
	gtk_widget_show (save1);
	gtk_container_add (GTK_CONTAINER (menu1), save1);

	save_as1 = gtk_image_menu_item_new_from_stock ("gtk-save-as", accel_group);
	gtk_widget_show (save_as1);
	gtk_container_add (GTK_CONTAINER (menu1), save_as1);

	separatormenuitem1 = gtk_separator_menu_item_new ();
	gtk_widget_show (separatormenuitem1);
	gtk_container_add (GTK_CONTAINER (menu1), separatormenuitem1);
	gtk_widget_set_sensitive (separatormenuitem1, FALSE);

	quit1 = gtk_image_menu_item_new_from_stock ("gtk-quit", accel_group);
	gtk_widget_show (quit1);
	gtk_container_add (GTK_CONTAINER (menu1), quit1);

	menuitem2 = gtk_menu_item_new_with_mnemonic ("_Activation Time");
	gtk_widget_show (menuitem2);
	gtk_container_add (GTK_CONTAINER (menubar1), menuitem2);

	menu2 = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem2), menu2);

	active1 = gtk_check_menu_item_new_with_label ("1 sec");
	gtk_widget_show (active1);
	gtk_container_add (GTK_CONTAINER (menu2), active1);

	active2 = gtk_check_menu_item_new_with_label ("2 secs");
	gtk_widget_show (active2);
	gtk_container_add (GTK_CONTAINER (menu2), active2);

	set_active_secs(3);
	active3 = gtk_check_menu_item_new_with_label ("3 secs");
	gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM(active3),
                                             TRUE);
	gtk_widget_show (active3);
	gtk_container_add (GTK_CONTAINER (menu2), active3);
	
	active4 = gtk_check_menu_item_new_with_label ("4 secs");
	gtk_widget_show (active4);
	gtk_container_add (GTK_CONTAINER (menu2), active4);

	active5 = gtk_check_menu_item_new_with_label ("5 secs");
	gtk_widget_show (active5);
	gtk_container_add (GTK_CONTAINER (menu2), active5);

	copy1 = gtk_image_menu_item_new_from_stock ("gtk-copy", accel_group);
	gtk_widget_show (copy1);
	//gtk_container_add (GTK_CONTAINER (menu2), copy1);

	paste1 = gtk_image_menu_item_new_from_stock ("gtk-paste", accel_group);
	gtk_widget_show (paste1);
	//gtk_container_add (GTK_CONTAINER (menu2), paste1);

	delete1 = gtk_image_menu_item_new_from_stock ("gtk-delete", accel_group);
	gtk_widget_show (delete1);
	//gtk_container_add (GTK_CONTAINER (menu2), delete1);

	menuitem3 = gtk_menu_item_new_with_mnemonic ("_View");
	gtk_widget_show (menuitem3);
	//gtk_container_add (GTK_CONTAINER (menubar1), menuitem3);

	menu3 = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem3), menu3);

	menuitem4 = gtk_menu_item_new_with_mnemonic ("_Help");
	gtk_widget_show (menuitem4);
	//gtk_container_add (GTK_CONTAINER (menubar1), menuitem4);

	menu4 = gtk_menu_new ();
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem4), menu4);

	about1 = gtk_menu_item_new_with_mnemonic ("_About");
	gtk_widget_show (about1);
	gtk_container_add (GTK_CONTAINER (menu4), about1);

	scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (scrolledwindow2),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_widget_show (scrolledwindow2);
	gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow2, TRUE, TRUE, 0);

	datamodel = create_model();

	/* create tree view */
	treeview = gtk_tree_view_new_with_model (datamodel);
	printf("i just set up my treeview\n");
	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (treeview), TRUE);

	g_object_unref (G_OBJECT (datamodel));

	gtk_container_add (GTK_CONTAINER (scrolledwindow2), treeview);
	add_columns();
	//add_columns (GTK_TREE_VIEW (treeview));
	



     /* finish & show */
      gtk_window_set_default_size (GTK_WINDOW (main_window), 500, 300);




  statusbar1 = gtk_statusbar_new ();
  gtk_widget_show (statusbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), statusbar1, FALSE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (new1), "activate",
                      GTK_SIGNAL_FUNC (on_new1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (open1), "activate",
                      GTK_SIGNAL_FUNC (on_open1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (save1), "activate",
                      GTK_SIGNAL_FUNC (on_save1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (save_as1), "activate",
                      GTK_SIGNAL_FUNC (on_save_as1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (quit1), "activate",
                      GTK_SIGNAL_FUNC (on_quit1_activate),
                      NULL);
		      /*
  gtk_signal_connect (GTK_OBJECT (cut1), "activate",
                      GTK_SIGNAL_FUNC (on_cut1_activate),
                      NULL);*/
  gtk_signal_connect (GTK_OBJECT (active1), "activate",
                      GTK_SIGNAL_FUNC (on_active1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (active2), "activate",
                      GTK_SIGNAL_FUNC (on_active2_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (active3), "activate",
                      GTK_SIGNAL_FUNC (on_active3_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (active4), "activate",
                      GTK_SIGNAL_FUNC (on_active4_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (active5), "activate",
                      GTK_SIGNAL_FUNC (on_active5_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (copy1), "activate",
                      GTK_SIGNAL_FUNC (on_copy1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (paste1), "activate",
                      GTK_SIGNAL_FUNC (on_paste1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (delete1), "activate",
                      GTK_SIGNAL_FUNC (on_delete1_activate),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (about1), "activate",
                      GTK_SIGNAL_FUNC (on_about1_activate),
                      NULL);

  /* Store pointers to all widgets, for use by lookup_widget(). */
  GLADE_HOOKUP_OBJECT_NO_REF (main_window, main_window, "main_window");
  GLADE_HOOKUP_OBJECT (main_window, viewport1, "viewport1");
  GLADE_HOOKUP_OBJECT (main_window, vbox1, "vbox1");
  GLADE_HOOKUP_OBJECT (main_window, menubar1, "menubar1");
  GLADE_HOOKUP_OBJECT (main_window, menuitem1, "menuitem1");
  GLADE_HOOKUP_OBJECT (main_window, menu1, "menu1");
  GLADE_HOOKUP_OBJECT (main_window, new1, "new1");
  GLADE_HOOKUP_OBJECT (main_window, open1, "open1");
  GLADE_HOOKUP_OBJECT (main_window, save1, "save1");
  GLADE_HOOKUP_OBJECT (main_window, save_as1, "save_as1");
  GLADE_HOOKUP_OBJECT (main_window, separatormenuitem1, "separatormenuitem1");
  GLADE_HOOKUP_OBJECT (main_window, quit1, "quit1");
  GLADE_HOOKUP_OBJECT (main_window, menuitem2, "menuitem2");
  GLADE_HOOKUP_OBJECT (main_window, menu2, "menu2");
  GLADE_HOOKUP_OBJECT (main_window, active1, "active1");
  GLADE_HOOKUP_OBJECT (main_window, active2, "active2");
  GLADE_HOOKUP_OBJECT (main_window, active3, "active3");
  GLADE_HOOKUP_OBJECT (main_window, active4, "active4");
  GLADE_HOOKUP_OBJECT (main_window, active5, "active5");
  GLADE_HOOKUP_OBJECT (main_window, copy1, "copy1");
  GLADE_HOOKUP_OBJECT (main_window, paste1, "paste1");
  GLADE_HOOKUP_OBJECT (main_window, delete1, "delete1");
  GLADE_HOOKUP_OBJECT (main_window, menuitem3, "menuitem3");
  GLADE_HOOKUP_OBJECT (main_window, menu3, "menu3");
  GLADE_HOOKUP_OBJECT (main_window, menuitem4, "menuitem4");
  GLADE_HOOKUP_OBJECT (main_window, menu4, "menu4");
  GLADE_HOOKUP_OBJECT (main_window, about1, "about1");
  GLADE_HOOKUP_OBJECT (main_window, scrolledwindow2, "scrolledwindow2");
  GLADE_HOOKUP_OBJECT (main_window, treeview, "main_list");
  GLADE_HOOKUP_OBJECT (main_window, statusbar1, "statusbar1");

  gtk_window_add_accel_group (GTK_WINDOW (main_window), accel_group);

    if (!GTK_WIDGET_VISIBLE (main_window))
    gtk_widget_show_all (main_window);
  else
    {
      gtk_widget_destroy (main_window);
      main_window = NULL;
    }

 	printf("I am creating a thread for deactivation method loop\n");
	if ((err = pthread_create(&threadId,NULL,deactivate,NULL))){
		printf("\n\nthread create failed %d\n\n",err);
		exit(1);
	}//end if

  return main_window;
}

